function [R,S]=conidaf(u,r,nr,ns,B,A,Hr,Hs,Fin,lam1,lam0)
% CONIDAF	is used to identify a discrete time controller of a plant operating in 
%		closed-loop (for controller order reduction) based on AF-CLOE method
%
%		[R,S]=conidaf(u,r,nr,ns,B,A,Hr,Hs,Fin,lam1,lam0)
%               
%		u and r are the column vectors containing respectively the output of the
%		controller and the excitation signal.
%
%		nr, ns are the order of the polynomials R and S.
% 
%		B and A are the numerator and denominator of the plant model (known).
%
%		Hr and Hs are the fixed terms of controller (robustness filter W=Hr/Hs)
%
%		Fin is the initial gain F0=Fin*(na+nb)*eye(na+nb) (Fin=1000 by default)
%
% 		lam1 and lam0 make different adaptation algoritms as follows:
%		
%		lam1=1;lam0=1   	 :decreasing gain (default algorithm)
%		0.95<lam1<1;lam0=1 :decreasing gain with fixed forgetting factor 
%		0.95<lam1,lam0<1	 :decreasing gain with variable forgetting factor 
%
% June 30,1999
% Copyright by Adaptech, 1997-1999

[nl,nc]=size(u);
if nc>2, error('This routine is only for SISO systems'),end
[nl,nc]=size(r);
if nc>2, error('This routine is only for SISO systems'),end
if (nr<0 | ns<0), error('The order of R and S should not be negative!!'),end

if nargin<6, error('This routin needs more parameters!'),end 
if nargin<7, lam1=1;lam0=1;Fin=1000;Hr=1;Hs=1;end
if nargin<8, lam1=1;lam0=1;Fin=1000;Hs=1;end
if nargin<9, lam1=1;lam0=1;Fin=1000;end
if nargin<10, lam1=1;lam0=1;end
if nargin<11, lam0=1;end

if isempty(Hr), Hr=1;end
if isempty(Hs), Hs=1;end
if isempty(Fin), Fin=1000;end
if isempty(lam1),lam1=1;end
if isempty(lam0), lam0=1;end

Hs1=Hs;Hr1=Hr;
if length(Hr)>length(Hs), Hs1=[Hs zeros(1,length(Hr)-length(Hs))];
else Hr1=[Hr zeros(1,length(Hs)-length(Hr))];end
B=conv(B,Hr1);
A=conv(A,Hs1);


B=B./A(1);A=A./A(1);
[rs,cs]=size(A);
[rr,cr]=size(B);
if min(size(B))>1, error('B should be a column vector!');end
if min(size(A))>1, error('A should be a column vector!');end
if cs>rs, A=A';end
if cr>rr, B=B';end


u=filter(Hs1,Hr1,u);

nd=min(length(r),length(u));	% number of data
nb=length(B);
na=length(A);
d=0;
for i=1:nb, if B(i)==0, d=d+1;else, break;end;end 
if d==0, error('B should contain at least one leading zero');end
nth=nr+ns+1;

nas=na+ns;
nbr=nb+nr;
if nas>nbr, zas=0;zbr=nas-nbr;else zas=nbr-nas;zbr=0;end

if nd<nth, error('Number of data should be greater than the number of parameters!'),end	

np=max(nas,nbr);


if (lam1>1 | lam0>1), error('lam1 and lam0 should be less than 1');end
if (lam1<0.95 | lam0<0.95), disp ('warning :lam1 and lam0 are normally greater than 0.95');end

P=[1;zeros(np-1,1)]';
theta=zeros(nth,1);
phi=zeros(2*np,1);
phi_f=zeros(2*np,1);
rt=[zeros(na+1,1)];
yhat=0;

F=Fin*nth*eye(nth);

i=[1:np-1 np+1:2*np-1];		%shift index for vector phi
j=[1:ns np+1:np+nr+1];     	%phi index for theta

for t=1:nd
   %Compute uhat 
   rt(2:na+1)=rt(1:na);rt(1)=r(t);
   if na>1, Au=A(2:na)'*phi(np+1:np+na-1);else, Au=0;end
   if d>1, uhat=B(d+1:nb)'*[phi(d-1:nb-2)]-Au+A'*rt(1:na);
   else uhat=B(d+1:nb)'*[-yhat;phi(d:nb-2)]-Au+A'*rt(1:na);
   end
   
	phi(i+1)=phi(i);phi(1)=-yhat;phi(np+1)=uhat;
   
   % Filtrage de phi 
   S=[1;theta(1:ns)];
   R=theta(ns+1:nth);
   P2=addz(conv(A,S),conv(B,R));
   if max(abs(roots(P2))) < 1,P=P2;end
   
	yuf=A'*[phi(1:na) phi(np+1:np+na)]-P(2:np)*[phi_f(1:np-1) phi_f(np+1:2*np-1)];
	phi_f(i+1)=phi_f(i);phi_f(1)=yuf(1);phi_f(np+1)=yuf(2);

      
   yhat=theta'*phi(j);
	e_apri=u(t)-yhat;
	e_apost=e_apri/(1+phi_f(j)'*F*phi_f(j));
	theta=theta+F*phi_f(j)*e_apost;

	F=1/lam1*(F-((F*phi_f(j)*phi_f(j)'*F)/(lam1+phi_f(j)'*F*phi_f(j))));
	lam1=lam0*lam1+1-lam0;

	yhat=theta'*phi(j);  

end
S=conv([1;theta(1:ns)]',Hs);
R=conv(theta(ns+1:nth)',Hr);
